@@ -15,6 +15,7 @@
 #define TEST_PRESERVE_HEADERS
 #define TEST_GET_BODY
 #define PRINT_MIME_STRUCT
+/*#define TEST_WRITE_TO_STREAM*/
 
 void
 print_depth (int depth)
@@ -26,39 +27,51 @@ print_depth (int depth)
 }
 
 void
-print_mime_struct (GMimePart *part, int depth)
+print_mime_struct (GMimeObject *part, int depth)
 {
 	const GMimeContentType *type;
+	GMimeObject *object;
 	
 	print_depth (depth);
-	type = g_mime_part_get_content_type (part);
+	type = g_mime_object_get_content_type (part);
 	fprintf (stdout, "Content-Type: %s/%s\n", type->type, type->subtype);
-	
-	if (g_mime_content_type_is_type (type, "multipart", "*")) {
-		GList *child;
 		
-		child = part->children;
-		while (child) {
-			print_mime_struct (child->data, depth + 1);
-			child = child->next;
+	if (GMIME_IS_MULTIPART (part)) {
+		GList *subpart;
+		
+		subpart = GMIME_MULTIPART (part)->subparts;
+		while (subpart) {
+			print_mime_struct (subpart->data, depth + 1);
+			subpart = subpart->next;
 		}
+	} else if (GMIME_IS_MESSAGE_PART (part)) {
+		GMimeMessagePart *mpart = (GMimeMessagePart *) part;
+		
+		if (mpart->message)
+			print_mime_struct (mpart->message->mime_part, depth + 1);
 	}
 }
 
 void
 test_parser (GMimeStream *stream)
 {
+	GMimeParser *parser;
 	GMimeMessage *message;
 	gboolean is_html;
 	char *text;
 	
 	fprintf (stdout, "\nTesting MIME parser...\n\n");
 	
+	parser = g_mime_parser_new ();
+	g_mime_parser_init_with_stream (parser, stream);
+	
 	ZenTimerStart();
-	message = g_mime_parser_construct_message (stream);
+	message = g_mime_parser_construct_message (parser);
 	ZenTimerStop();
 	ZenTimerReport ("gmime::parser_construct_message");
 	
+	g_object_unref (parser);
+	
 	ZenTimerStart();
 	text = g_mime_message_to_string (message);
 	ZenTimerStop();
@@ -72,7 +85,7 @@ test_parser (GMimeStream *stream)
 		
 		fprintf (stdout, "\nTesting preservation of headers...\n\n");
 		stream = g_mime_stream_file_new (stdout);
-		g_mime_header_write_to_stream (message->header->headers, stream);
+		g_mime_header_write_to_stream (GMIME_OBJECT (message)->headers, stream);
 		g_mime_stream_flush (stream);
 		GMIME_STREAM_FILE (stream)->fp = NULL;
 		g_mime_stream_unref (stream);
@@ -94,6 +107,14 @@ test_parser (GMimeStream *stream)
 	}
 #endif
 	
+#ifdef TEST_WRITE_TO_STREAM
+	stream = g_mime_stream_fs_new (2);
+	g_mime_object_write_to_stream (GMIME_OBJECT (message), stream);
+	g_mime_stream_flush (stream);
+	GMIME_STREAM_FS (stream)->fd = -1;
+	g_mime_stream_unref (stream);
+#endif
+	
 #ifdef PRINT_MIME_STRUCT
 	/* print mime structure */
 	print_mime_struct (message->mime_part, 0);
@@ -108,15 +129,15 @@ test_parser (GMimeStream *stream)
 /*#define STREAM_BUFFER*/
 /*#define STREAM_MEM*/
 /*#define STREAM_MMAP*/
-#define CRLF_FILTER
 
 int main (int argc, char **argv)
 {
 	char *filename = NULL;
 	GMimeStream *stream, *istream;
-	GMimeFilter *filter;
 	int fd;
 	
+	g_mime_init (0);
+	
 	if (argc > 1)
 		filename = argv[1];
 	else
@@ -126,8 +147,6 @@ int main (int argc, char **argv)
 	if (fd == -1)
 		return 0;
 	
-	g_mime_init (GMIME_INIT_FLAG_UTF8);
-	
 #ifdef STREAM_MMAP
 	stream = g_mime_stream_mmap_new (fd, PROT_READ, MAP_PRIVATE);
 	g_assert (stream != NULL);
@@ -150,14 +169,6 @@ int main (int argc, char **argv)
 	stream = istream;
 #endif
 	
-#ifdef CRLF_FILTER
-	istream = g_mime_stream_filter_new_with_stream (stream);
-	filter = g_mime_filter_crlf_new (GMIME_FILTER_CRLF_DECODE, GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (istream), filter);
-	g_mime_stream_unref (stream);
-	stream = istream;
-#endif
-	
 	test_parser (stream);
 	
 	g_mime_stream_unref (stream);
